@@ -446,7 +446,6 @@ gdk_pixbuf_loader_write (GdkPixbufLoader *loader,
 {
         GdkPixbufLoaderPrivate *priv;
   
-        g_return_val_if_fail (loader != NULL, FALSE);
         g_return_val_if_fail (GDK_IS_PIXBUF_LOADER (loader), FALSE);
   
         g_return_val_if_fail (buf != NULL, FALSE);
@@ -501,7 +500,7 @@ gdk_pixbuf_loader_new (void)
 /**
  * gdk_pixbuf_loader_new_with_type:
  * @image_type: name of the image format to be loaded with the image
- * @error: return location for an allocated #GError, or %NULL to ignore errors
+ * @error: (allow-none): return location for an allocated #GError, or %NULL to ignore errors
  *
  * Creates a new pixbuf loader object that always attempts to parse
  * image data as if it were an image of type @image_type, instead of
@@ -544,7 +543,7 @@ gdk_pixbuf_loader_new_with_type (const char *image_type,
 /**
  * gdk_pixbuf_loader_new_with_mime_type:
  * @mime_type: the mime type to be loaded 
- * @error: return location for an allocated #GError, or %NULL to ignore errors
+ * @error: (allow-none): return location for an allocated #GError, or %NULL to ignore errors
  *
  * Creates a new pixbuf loader object that always attempts to parse
  * image data as if it were an image of mime type @mime_type, instead of
@@ -632,7 +631,6 @@ gdk_pixbuf_loader_get_pixbuf (GdkPixbufLoader *loader)
 {
         GdkPixbufLoaderPrivate *priv;
   
-        g_return_val_if_fail (loader != NULL, NULL);
         g_return_val_if_fail (GDK_IS_PIXBUF_LOADER (loader), NULL);
   
         priv = loader->priv;
@@ -661,7 +659,6 @@ gdk_pixbuf_loader_get_animation (GdkPixbufLoader *loader)
 {
         GdkPixbufLoaderPrivate *priv;
   
-        g_return_val_if_fail (loader != NULL, NULL);
         g_return_val_if_fail (GDK_IS_PIXBUF_LOADER (loader), NULL);
   
         priv = loader->priv;
@@ -672,7 +669,7 @@ gdk_pixbuf_loader_get_animation (GdkPixbufLoader *loader)
 /**
  * gdk_pixbuf_loader_close:
  * @loader: A pixbuf loader.
- * @error: return location for a #GError, or %NULL to ignore errors
+ * @error: (allow-none): return location for a #GError, or %NULL to ignore errors
  *
  * Informs a pixbuf loader that no further writes with
  * gdk_pixbuf_loader_write() will occur, so that it can free its
@@ -694,7 +691,6 @@ gdk_pixbuf_loader_close (GdkPixbufLoader *loader,
         GdkPixbufLoaderPrivate *priv;
         gboolean retval = TRUE;
   
-        g_return_val_if_fail (loader != NULL, TRUE);
         g_return_val_if_fail (GDK_IS_PIXBUF_LOADER (loader), TRUE);
         g_return_val_if_fail (error == NULL || *error == NULL, TRUE);
   
@@ -773,7 +769,6 @@ gdk_pixbuf_loader_get_format (GdkPixbufLoader *loader)
 {
         GdkPixbufLoaderPrivate *priv;
   
-        g_return_val_if_fail (loader != NULL, NULL);
         g_return_val_if_fail (GDK_IS_PIXBUF_LOADER (loader), NULL);
   
         priv = loader->priv;
