@@ -102,9 +102,6 @@ static gboolean is_modally_blocked (GdkWindow   *window);
 
 static GList *client_filters;	/* Filters for client messages */
 
-static gboolean p_grab_automatic;
-static GdkEventMask p_grab_mask;
-static gboolean p_grab_owner_events, k_grab_owner_events;
 static HCURSOR p_grab_cursor;
 
 static GSourceFuncs event_funcs = {
@@ -262,7 +259,7 @@ inner_window_procedure (HWND   hwnd,
   else
     {
       /* Otherwise call DefWindowProcW(). */
-      GDK_NOTE (EVENTLOOP, g_print (" DefWindowProcW"));
+      GDK_NOTE (EVENTS, g_print (" DefWindowProcW"));
       return DefWindowProcW (hwnd, message, wparam, lparam);
     }
 }
@@ -429,6 +426,8 @@ gdk_event_get_graphics_expose (GdkWindow *window)
   return NULL;	
 }
 
+#if 0 /* Unused, but might be useful to re-introduce in some debugging output? */
+
 static char *
 event_mask_string (GdkEventMask mask)
 {
@@ -465,6 +464,8 @@ event_mask_string (GdkEventMask mask)
   return bfr;
 }
 
+#endif
+
 GdkGrabStatus
 _gdk_windowing_pointer_grab (GdkWindow    *window,
 			     GdkWindow    *native_window,
@@ -474,9 +475,52 @@ _gdk_windowing_pointer_grab (GdkWindow    *window,
 			     GdkCursor    *cursor,
 			     guint32	time)
 {
-  SetCapture (GDK_WINDOW_HWND (native_window));
-  /* TODO_CSW: grab brokens, confine window, cursor, input_grab */
-  return GDK_GRAB_SUCCESS;
+  HCURSOR hcursor;
+  GdkCursorPrivate *cursor_private;
+  gint return_val;
+
+  g_return_val_if_fail (window != NULL, 0);
+  g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
+  g_return_val_if_fail (confine_to == NULL || GDK_IS_WINDOW (confine_to), 0);
+  
+  cursor_private = (GdkCursorPrivate*) cursor;
+  
+  if (!cursor)
+    hcursor = NULL;
+  else if ((hcursor = CopyCursor (cursor_private->hcursor)) == NULL)
+    WIN32_API_FAILED ("CopyCursor");
+
+  return_val = _gdk_input_grab_pointer (native_window,
+					owner_events,
+					event_mask,
+					confine_to,
+					time);
+
+  if (return_val == GDK_GRAB_SUCCESS)
+    {
+      GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (((GdkWindowObject *) native_window)->impl);
+
+      SetCapture (GDK_WINDOW_HWND (native_window));
+      /* TODO_CSW: grab brokens, confine window, input_grab */
+      if (p_grab_cursor != NULL)
+	{
+	  if (GetCursor () == p_grab_cursor)
+	    SetCursor (NULL);
+	  DestroyCursor (p_grab_cursor);
+	}
+
+      p_grab_cursor = hcursor;
+
+      if (p_grab_cursor != NULL)
+	SetCursor (p_grab_cursor);
+      else if (impl->hcursor != NULL)
+	SetCursor (impl->hcursor);
+      else
+	SetCursor (LoadCursor (NULL, IDC_ARROW));
+
+    }
+
+  return return_val;
 }
 
 void
@@ -730,8 +774,8 @@ print_event_state (guint state)
 #undef CASE
 }
 
-static void
-print_event (const GdkEvent *event)
+void
+_gdk_win32_print_event (const GdkEvent *event)
 {
   gchar *escaped, *kvname;
   gchar *selection_name, *target_name, *property_name;
@@ -781,7 +825,7 @@ print_event (const GdkEvent *event)
     default: g_assert_not_reached ();
     }
 
-  g_print (" %p ", GDK_WINDOW_HWND (event->any.window));
+  g_print (" %p ", event->any.window ? GDK_WINDOW_HWND (event->any.window) : NULL);
 
   switch (event->any.type)
     {
@@ -845,9 +889,14 @@ print_event (const GdkEvent *event)
     case GDK_FOCUS_CHANGE:
       g_print ("%s", (event->focus_change.in ? "IN" : "OUT"));
       break;
+    case GDK_CONFIGURE:
+      g_print ("x:%d y:%d w:%d h:%d",
+	       event->configure.x, event->configure.y,
+	       event->configure.width, event->configure.height);
+      break;
+    case GDK_SELECTION_CLEAR:
     case GDK_SELECTION_REQUEST:
     case GDK_SELECTION_NOTIFY:
-    case GDK_SELECTION_CLEAR:
       selection_name = gdk_atom_name (event->selection.selection);
       target_name = gdk_atom_name (event->selection.target);
       property_name = gdk_atom_name (event->selection.property);
@@ -857,10 +906,19 @@ print_event (const GdkEvent *event)
       g_free (target_name);
       g_free (property_name);
       break;
-    case GDK_CONFIGURE:
-      g_print ("x:%d y:%d w:%d h:%d",
-	       event->configure.x, event->configure.y,
-	       event->configure.width, event->configure.height);
+    case GDK_DRAG_ENTER:
+    case GDK_DRAG_LEAVE:
+    case GDK_DRAG_MOTION:
+    case GDK_DRAG_STATUS:
+    case GDK_DROP_START:
+    case GDK_DROP_FINISHED:
+      if (event->dnd.context != NULL)
+	g_print ("ctx:%p: %s %s src:%p dest:%p",
+		 event->dnd.context,
+		 _gdk_win32_drag_protocol_to_string (event->dnd.context->protocol),
+		 event->dnd.context->is_source ? "SOURCE" : "DEST",
+		 event->dnd.context->source_window == NULL ? NULL : GDK_WINDOW_HWND (event->dnd.context->source_window),
+		 event->dnd.context->dest_window == NULL ? NULL : GDK_WINDOW_HWND (event->dnd.context->dest_window));
       break;
     case GDK_CLIENT_EVENT:
       g_print ("%s %d %ld %ld %ld %ld %ld",
@@ -947,11 +1005,12 @@ append_event (GdkEvent *event)
   fixup_event (event);
 #if 1
   link = _gdk_event_queue_append (_gdk_display, event);
+  GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
   /* event morphing, the passed in may not be valid afterwards */
   _gdk_windowing_got_event (_gdk_display, link, event, 0);
 #else
   _gdk_event_queue_append (_gdk_display, event);
-  GDK_NOTE (EVENTS, print_event (event));
+  GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
 #endif
 }
 
@@ -1064,7 +1123,7 @@ apply_event_filters (GdkWindow  *window,
     {
       ((GdkEventPrivate *)event)->flags &= ~GDK_EVENT_PENDING;
       fixup_event (event);
-      GDK_NOTE (EVENTS, print_event (event));
+      GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
     }
   return result;
 }
@@ -1159,17 +1218,6 @@ do_show_window (GdkWindow *window, gboolean hide_window)
     }
 }
 
-static gboolean
-gdk_window_is_ancestor (GdkWindow *ancestor,
-			GdkWindow *window)
-{
-  if (ancestor == NULL || window == NULL)
-    return FALSE;
-
-  return (gdk_window_get_parent (window) == ancestor ||
-	  gdk_window_is_ancestor (ancestor, gdk_window_get_parent (window)));
-}
-
 static void
 synthesize_enter_or_leave_event (GdkWindow    	*window,
 				 MSG          	*msg,
@@ -1299,20 +1347,6 @@ update_colors (GdkWindow *window,
   GDK_NOTE (COLORMAP, (top ? g_print ("\n") : (void) 0));
 }
 
-static void
-translate_mouse_coords (GdkWindow *window1,
-			GdkWindow *window2,
-			MSG       *msg)
-{
-  POINT pt;
-
-  pt.x = GET_X_LPARAM (msg->lParam);
-  pt.y = GET_Y_LPARAM (msg->lParam);
-  ClientToScreen (GDK_WINDOW_HWND (window1), &pt);
-  ScreenToClient (GDK_WINDOW_HWND (window2), &pt);
-  msg->lParam = MAKELPARAM (pt.x, pt.y);
-}
-
 /* The check_extended flag controls whether to check if the windows want
  * events from extended input devices and if the message should be skipped
  * because an extended input device is active
@@ -1432,43 +1466,6 @@ doesnt_want_char (gint mask,
   return !(mask & (GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK));
 }
 
-static gboolean
-doesnt_want_button_press (gint mask,
-			  MSG *msg)
-{
-  return !(mask & GDK_BUTTON_PRESS_MASK);
-}
-
-static gboolean
-doesnt_want_button_release (gint mask,
-			    MSG *msg)
-{
-  return !(mask & GDK_BUTTON_RELEASE_MASK);
-}
-
-static gboolean
-doesnt_want_button_motion (gint mask,
-			   MSG *msg)
-{
-  return !((mask & GDK_POINTER_MOTION_MASK) ||
-	   ((msg->wParam & (MK_LBUTTON|MK_MBUTTON|MK_RBUTTON)) && (mask & GDK_BUTTON_MOTION_MASK)) ||
-	   ((msg->wParam & MK_LBUTTON) && (mask & GDK_BUTTON1_MOTION_MASK)) ||
-	   ((msg->wParam & MK_MBUTTON) && (mask & GDK_BUTTON2_MOTION_MASK)) ||
-	   ((msg->wParam & MK_RBUTTON) && (mask & GDK_BUTTON3_MOTION_MASK)));
-}
-
-static gboolean
-doesnt_want_scroll (gint mask,
-		    MSG *msg)
-{
-  /* As there are no separate scroll events in X11, button press
-   * events are used, so higher level code might be selecting for
-   * either GDK_BUTTON_PRESS_MASK or GDK_SCROLL_MASK when it wants GDK
-   * scroll events. Make sure this works in the Win32 backend, too.
-   */
-  return !(mask & (GDK_SCROLL_MASK|GDK_BUTTON_PRESS_MASK));
-}
-
 static void
 handle_configure_event (MSG       *msg,
 			GdkWindow *window)
@@ -1645,22 +1642,44 @@ handle_wm_paint (MSG        *msg,
   DeleteObject (hrgn);
 }
 
-static void
-handle_stuff_while_moving_or_resizing (void)
-{
-  int arbitrary_limit = 1;
-  while (g_main_context_pending (NULL) && arbitrary_limit--)
-    g_main_context_iteration (NULL, FALSE);
-}
-
-static VOID CALLBACK
+static VOID CALLBACK 
 modal_timer_proc (HWND     hwnd,
 		  UINT     msg,
 		  UINT_PTR id,
 		  DWORD    time)
 {
-  if (_sizemove_in_progress)
-    handle_stuff_while_moving_or_resizing ();
+  int arbitrary_limit = 1;
+
+  while (_modal_operation_in_progress &&
+	 g_main_context_pending (NULL) &&
+	 arbitrary_limit--)
+    g_main_context_iteration (NULL, FALSE);
+}
+
+void
+_gdk_win32_begin_modal_call (void)
+{
+  g_assert (!_modal_operation_in_progress);
+
+  _modal_operation_in_progress = TRUE;
+
+  modal_timer = SetTimer (NULL, 0, 10, modal_timer_proc);
+  if (modal_timer == 0)
+    WIN32_API_FAILED ("SetTimer");
+}
+
+void
+_gdk_win32_end_modal_call (void)
+{
+  g_assert (_modal_operation_in_progress);
+
+  _modal_operation_in_progress = FALSE;
+
+  if (modal_timer != 0)
+    {
+      API_CALL (KillTimer, (NULL, modal_timer));
+      modal_timer = 0;
+   }
 }
 
 static VOID CALLBACK
@@ -1875,8 +1894,6 @@ gdk_event_translate (MSG  *msg,
 
   GdkPointerGrabInfo *grab = NULL;
   GdkWindow *grab_window = NULL;
-  guint grab_mask = 0;
-  gboolean grab_owner_events = FALSE;
 
   static gint update_colors_counter = 0;
   gint button;
@@ -2001,7 +2018,7 @@ gdk_event_translate (MSG  *msg,
 
 	case GDK_FILTER_TRANSLATE:
 	  ((GdkEventPrivate *)event)->flags &= ~GDK_EVENT_PENDING;
-	  GDK_NOTE (EVENTS, print_event (event));
+	  GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
 	  return_val = TRUE;
 	  goto done;
 
@@ -2015,7 +2032,7 @@ gdk_event_translate (MSG  *msg,
 	  event->client.data.l[0] = msg->lParam;
 	  for (i = 1; i < 5; i++)
 	    event->client.data.l[i] = 0;
-	  GDK_NOTE (EVENTS, print_event (event));
+	  GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
 	  return_val = TRUE;
 	  goto done;
 	}
@@ -2721,23 +2738,13 @@ gdk_event_translate (MSG  *msg,
       break;
 
     case WM_ENTERSIZEMOVE:
-      _sizemove_in_progress = TRUE;
-      modal_timer = SetTimer (NULL, 0, 20, modal_timer_proc);
+    case WM_ENTERMENULOOP:
+      _gdk_win32_begin_modal_call ();
       break;
 
     case WM_EXITSIZEMOVE:
-      _sizemove_in_progress = FALSE;
-      KillTimer (NULL, modal_timer);
-      break;
-
-    case WM_ENTERMENULOOP:
-      _sizemove_in_progress = TRUE;
-      modal_timer = SetTimer (NULL, 0, 20, modal_timer_proc);
-      break;
-
     case WM_EXITMENULOOP:
-      _sizemove_in_progress = FALSE;
-      KillTimer (NULL, modal_timer);
+      _gdk_win32_end_modal_call ();
       break;
 
     case WM_WINDOWPOSCHANGING:
@@ -2770,7 +2777,7 @@ gdk_event_translate (MSG  *msg,
 				 windowpos->cx, windowpos->cy, windowpos->x, windowpos->y));
 
       /* If position and size haven't changed, don't do anything */
-      if (_sizemove_in_progress &&
+      if (_modal_operation_in_progress &&
 	  (windowpos->flags & SWP_NOMOVE) &&
 	  (windowpos->flags & SWP_NOSIZE))
 	break;
@@ -2778,7 +2785,7 @@ gdk_event_translate (MSG  *msg,
       /* Once we've entered the moving or sizing modal loop, we won't
        * return to the main loop until we're done sizing or moving.
        */
-      if (_sizemove_in_progress &&
+      if (_modal_operation_in_progress &&
 	 GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
 	 !GDK_WINDOW_DESTROYED (window))
 	{
@@ -2787,13 +2794,13 @@ gdk_event_translate (MSG  *msg,
 	      GDK_NOTE (EVENTS, g_print (" do magic"));
 	      if (((GdkWindowObject *) window)->resize_count > 1)
 		((GdkWindowObject *) window)->resize_count -= 1;
-	      
+
 	      handle_configure_event (msg, window);
 	      g_main_context_iteration (NULL, FALSE);
-
+#if 0
 	      /* Dispatch main loop - to realize resizes... */
-	      handle_stuff_while_moving_or_resizing ();
-	      
+	      modal_timer_proc (msg->hwnd, msg->message, 0, msg->time);
+#endif
 	      /* Claim as handled, so that WM_SIZE and WM_MOVE are avoided */
 	      return_val = TRUE;
 	      *ret_valp = 1;
@@ -3176,7 +3183,7 @@ gdk_event_translate (MSG  *msg,
 
 	  fixup_event (event);
 	  GDK_NOTE (EVENTS, g_print (" (calling gdk_event_func)"));
-	  GDK_NOTE (EVENTS, print_event (event));
+	  GDK_NOTE (EVENTS, _gdk_win32_print_event (event));
 	  (*_gdk_event_func) (event, _gdk_event_data);
 	  gdk_event_free (event);
 
@@ -3371,6 +3378,14 @@ gdk_event_dispatch (GSource     *source,
 	(*_gdk_event_func) (event, _gdk_event_data);
       
       gdk_event_free (event);
+
+      /* Do drag & drop if it is still pending */
+      if (_dnd_source_state == GDK_WIN32_DND_PENDING) 
+	{
+	  _dnd_source_state = GDK_WIN32_DND_DRAGGING;
+	  _gdk_win32_dnd_do_dragdrop ();
+	  _dnd_source_state = GDK_WIN32_DND_NONE;
+	}
     }
   
   GDK_THREADS_LEAVE ();
