@@ -25,7 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "gdk/gdkkeysyms.h"
@@ -152,7 +152,7 @@ gtk_socket_class_init (GtkSocketClass *class)
    * added to the socket. 
    */
   socket_signals[PLUG_ADDED] =
-    g_signal_new (I_("plug_added"),
+    g_signal_new (I_("plug-added"),
 		  G_OBJECT_CLASS_TYPE (class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkSocketClass, plug_added),
@@ -171,7 +171,7 @@ gtk_socket_class_init (GtkSocketClass *class)
    * Return value: %TRUE to stop other handlers from being invoked.
    */
   socket_signals[PLUG_REMOVED] =
-    g_signal_new (I_("plug_removed"),
+    g_signal_new (I_("plug-removed"),
 		  G_OBJECT_CLASS_TYPE (class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkSocketClass, plug_removed),
@@ -238,7 +238,7 @@ gtk_socket_steal (GtkSocket      *socket,
   g_return_if_fail (GTK_IS_SOCKET (socket));
   g_return_if_fail (GTK_WIDGET_ANCHORED (socket));
 
-  if (!GTK_WIDGET_REALIZED (socket))
+  if (!gtk_widget_get_realized (GTK_WIDGET (socket)))
     gtk_widget_realize (GTK_WIDGET (socket));
 
   _gtk_socket_add_window (socket, wid, TRUE);
@@ -269,7 +269,7 @@ gtk_socket_add_id (GtkSocket      *socket,
   g_return_if_fail (GTK_IS_SOCKET (socket));
   g_return_if_fail (GTK_WIDGET_ANCHORED (socket));
 
-  if (!GTK_WIDGET_REALIZED (socket))
+  if (!gtk_widget_get_realized (GTK_WIDGET (socket)))
     gtk_widget_realize (GTK_WIDGET (socket));
 
   _gtk_socket_add_window (socket, window_id, TRUE);
@@ -294,12 +294,31 @@ gtk_socket_get_id (GtkSocket *socket)
   g_return_val_if_fail (GTK_IS_SOCKET (socket), 0);
   g_return_val_if_fail (GTK_WIDGET_ANCHORED (socket), 0);
 
-  if (!GTK_WIDGET_REALIZED (socket))
+  if (!gtk_widget_get_realized (GTK_WIDGET (socket)))
     gtk_widget_realize (GTK_WIDGET (socket));
 
   return _gtk_socket_windowing_get_id (socket);
 }
 
+/**
+ * gtk_socket_get_plug_window:
+ * @socket_: a #GtkSocket.
+ *
+ * Retrieves the window of the plug. Use this to check if the plug has
+ * been created inside of the socket.
+ *
+ * Return value: the window of the plug if available, or %NULL
+ *
+ * Since:  2.14
+ **/
+GdkWindow*
+gtk_socket_get_plug_window (GtkSocket *socket)
+{
+  g_return_val_if_fail (GTK_IS_SOCKET (socket), NULL);
+
+  return socket->plug_window;
+}
+
 static void
 gtk_socket_realize (GtkWidget *widget)
 {
@@ -307,7 +326,7 @@ gtk_socket_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
 
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.x = widget->allocation.x;
@@ -334,8 +353,6 @@ gtk_socket_realize (GtkWidget *widget)
 			 _gtk_socket_windowing_filter_func,
 			 widget);
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
   /* We sync here so that we make sure that if the XID for
    * our window is passed to another application, SubstructureRedirectMask
    * will be set by the time the other app creates its window.
@@ -355,7 +372,6 @@ _gtk_socket_end_embedding (GtkSocket *socket)
 {
   GtkSocketPrivate *private = _gtk_socket_get_private (socket);
   GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (socket));
-  gint i;
   
   if (GTK_IS_WINDOW (toplevel))
     _gtk_socket_windowing_end_embedding_toplevel (socket);
@@ -366,12 +382,7 @@ _gtk_socket_end_embedding (GtkSocket *socket)
   socket->current_height = 0;
   private->resize_count = 0;
 
-  /* Remove from end to avoid indexes shifting. This is evil */
-  for (i = socket->accel_group->n_accels - 1; i >= 0; i--)
-    {
-      GtkAccelGroupEntry *accel_entry = &socket->accel_group->priv_accels[i];
-      gtk_accel_group_disconnect (socket->accel_group, accel_entry->closure);
-    }
+  gtk_accel_group_disconnect (socket->accel_group, NULL);
 }
 
 static void
@@ -379,7 +390,7 @@ gtk_socket_unrealize (GtkWidget *widget)
 {
   GtkSocket *socket = GTK_SOCKET (widget);
 
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, FALSE);
 
   if (socket->plug_widget)
     {
@@ -390,11 +401,10 @@ gtk_socket_unrealize (GtkWidget *widget)
       _gtk_socket_end_embedding (socket);
     }
 
-  if (GTK_WIDGET_CLASS (gtk_socket_parent_class)->unrealize)
-    (* GTK_WIDGET_CLASS (gtk_socket_parent_class)->unrealize) (widget);
+  GTK_WIDGET_CLASS (gtk_socket_parent_class)->unrealize (widget);
 }
-  
-static void 
+
+static void
 gtk_socket_size_request (GtkWidget      *widget,
 			 GtkRequisition *requisition)
 {
@@ -429,7 +439,7 @@ gtk_socket_size_allocate (GtkWidget     *widget,
   GtkSocket *socket = GTK_SOCKET (widget);
 
   widget->allocation = *allocation;
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     {
       gdk_window_move_resize (widget->window,
 			      allocation->x, allocation->y,
@@ -578,22 +588,9 @@ _gtk_socket_remove_grabbed_key (GtkSocket      *socket,
 				guint           keyval,
 				GdkModifierType modifiers)
 {
-  gint i;
-
-  for (i = 0; i < socket->accel_group->n_accels; i++)
-    {
-      GtkAccelGroupEntry *accel_entry = &socket->accel_group->priv_accels[i];
-      if (accel_entry->key.accel_key == keyval &&
-	  accel_entry->key.accel_mods == modifiers)
-	{
-	  gtk_accel_group_disconnect (socket->accel_group,
-				      accel_entry->closure);
-	  return;
-	}
-    }
-
-  g_warning ("GtkSocket: request to remove non-present grabbed key %u,%#x\n",
-	     keyval, modifiers);
+  if (!gtk_accel_group_disconnect_key (socket->accel_group, keyval, modifiers))
+    g_warning ("GtkSocket: request to remove non-present grabbed key %u,%#x\n",
+	       keyval, modifiers);
 }
 
 static void
@@ -605,7 +602,7 @@ socket_update_focus_in (GtkSocket *socket)
     {
       GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (socket));
 
-      if (GTK_WIDGET_TOPLEVEL (toplevel) &&
+      if (gtk_widget_is_toplevel (toplevel) &&
 	  GTK_WINDOW (toplevel)->has_toplevel_focus &&
 	  gtk_widget_is_focus (GTK_WIDGET (socket)))
 	focus_in = TRUE;
@@ -628,7 +625,7 @@ socket_update_active (GtkSocket *socket)
     {
       GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (socket));
 
-      if (GTK_WIDGET_TOPLEVEL (toplevel) &&
+      if (gtk_widget_is_toplevel (toplevel) &&
 	  GTK_WINDOW (toplevel)->is_active)
 	active = TRUE;
     }
@@ -696,7 +693,7 @@ gtk_socket_key_event (GtkWidget   *widget,
 {
   GtkSocket *socket = GTK_SOCKET (widget);
   
-  if (GTK_WIDGET_HAS_FOCUS (socket) && socket->plug_window && !socket->plug_widget)
+  if (gtk_widget_has_focus (widget) && socket->plug_window && !socket->plug_widget)
     {
       _gtk_socket_windowing_send_key_event (socket, (GdkEvent *) event, FALSE);
 
@@ -727,14 +724,16 @@ void
 _gtk_socket_claim_focus (GtkSocket *socket,
 			 gboolean   send_event)
 {
+  GtkWidget *widget = GTK_WIDGET (socket);
+
   if (!send_event)
     socket->focus_in = TRUE;	/* Otherwise, our notify handler will send FOCUS_IN  */
       
   /* Oh, the trickery... */
   
-  GTK_WIDGET_SET_FLAGS (socket, GTK_CAN_FOCUS);
-  gtk_widget_grab_focus (GTK_WIDGET (socket));
-  GTK_WIDGET_UNSET_FLAGS (socket, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (widget, TRUE);
+  gtk_widget_grab_focus (widget);
+  gtk_widget_set_can_focus (widget, FALSE);
 }
 
 static gboolean
@@ -953,7 +952,7 @@ _gtk_socket_advance_toplevel_focus (GtkSocket        *socket,
   if (!toplevel)
     return;
 
-  if (!GTK_WIDGET_TOPLEVEL (toplevel) || GTK_IS_PLUG (toplevel))
+  if (!gtk_widget_is_toplevel (toplevel) || GTK_IS_PLUG (toplevel))
     {
       gtk_widget_child_focus (toplevel,direction);
       return;
